@@ -11,11 +11,16 @@ from invoices.models import Arrival, Sale
 class InvoiceAdmin(admin.ModelAdmin):
 
     list_display = [
-        'get_name', 'type', 'customer', 'get_total_qty', 'get_total',
+        'get_name',
+        'type',
+        'customer',
+        'creator',
+        'get_total_qty',
+        'get_total',
         'get_item_actions'
     ]
     list_display_links = ['type']
-    list_filter = ['type', 'created']
+    list_filter = ['type', 'creator', 'created']
     search_fields = ['id']
 
     def has_add_permission(self, request):
