@@ -87,7 +87,7 @@ class SetDiscountForm(forms.Form):
 
 
 class SetQtyForm(forms.Form):
-    value = forms.IntegerField()
+    value = forms.FloatField()
 
 
 class SetPriceForm(forms.Form):
@@ -96,7 +96,7 @@ class SetPriceForm(forms.Form):
 
 class AddProductForm(forms.ModelForm):
 
-    stock = forms.IntegerField(label=_('Stock'), min_value=1)
+    stock = forms.FloatField(label=_('Stock'), min_value=1)
 
     class Meta:
         model = Product
