@@ -1,12 +1,15 @@
 
+from datetime import datetime, time
+
 from django.apps import apps
-
+from django.utils.functional import cached_property
 from django.urls import reverse_lazy
 from django.core.exceptions import ObjectDoesNotExist
 from django.db import models, transaction
 from django.utils import timezone
 from django.utils.translation import ugettext_lazy as _
 from django.template.loader import render_to_string
+from django.contrib.auth import get_user_model
 
 from customers.models import CustomerField
 
@@ -58,6 +61,11 @@ class Invoice(models.Model):
     type = NotImplemented
     customer = NotImplemented
 
+    creator = models.ForeignKey(
+        get_user_model(),
+        verbose_name=_('Creator'),
+        on_delete=models.PROTECT)
+
     created = models.DateTimeField(_('Creation date'), auto_now_add=True)
 
     discount = models.PositiveIntegerField(
@@ -82,6 +90,12 @@ class Invoice(models.Model):
     def create(cls, type):
         return cls.objects.create(type=type)
 
+    @property
+    def customer_name(self):
+        if self.customer:
+            return self.customer.name
+        return ''
+
     @property
     def invoice_type(self):
         return self.__class__.__name__.lower()
@@ -117,7 +131,7 @@ class Invoice(models.Model):
         return self._meta.verbose_name
 
     @transaction.atomic
-    def add_item(self, product, qty=1):
+    def add_item(self, product, qty=1, update_product=True):
         try:
             item = self.items.get(product=product)
             item.qty += qty
@@ -130,11 +144,11 @@ class Invoice(models.Model):
                 **product.price_values
             )
 
-        self._handle_add_item(product, qty)
+        self._handle_add_item(product, qty, update_product=update_product)
 
         return item
 
-    def _handle_add_item(self, product, qty):
+    def _handle_add_item(self, product, qty, update_product):
         pass
 
     @transaction.atomic
@@ -160,9 +174,7 @@ class Invoice(models.Model):
 
             item.initial_currency = CURRENCY_UAH
             item.price_retail = value
-            item.save(update_fields=['price_retail'])
-
-            ExchangeRates.update_prices(self.items.filter(pk=item.pk))
+            item.save()
 
         return item
 
@@ -253,12 +265,19 @@ class InvoiceItem(MultiCurrencyPrice):
         verbose_name=_('Product'),
         on_delete=models.CASCADE)
 
-    qty = models.IntegerField(_('Quantity'))
+    qty = models.FloatField(_('Quantity'))
+
+    def get_qty_input_value(self):
+        return str(self.qty).replace(',', '')
 
     def set_rates(self, rates):
         self.rates = rates
 
-    @property
+    @cached_property
+    def customer_name(self):
+        return self.invoice.customer_name
+
+    @cached_property
     def product_name(self):
         return self.product.name
 
@@ -371,9 +390,10 @@ class Arrival(Invoice):
 
     customer = CustomerField(related_name='arrivals')
 
-    def _handle_add_item(self, product, qty):
+    def _handle_add_item(self, product, qty, update_product):
 
-        product.add_stock(value=qty)
+        if update_product:
+            product.add_stock(value=qty)
 
     def _handle_set_item_qty(self, item, value):
 
@@ -419,9 +439,10 @@ class Sale(Invoice):
 
     customer = CustomerField(related_name='sales')
 
-    def _handle_add_item(self, product, qty):
+    def _handle_add_item(self, product, qty, update_product):
 
-        product.subtract_stock(value=qty)
+        if update_product:
+            product.subtract_stock(value=qty)
 
     def _handle_set_item_qty(self, item, value):
 
@@ -466,7 +487,7 @@ class InvoicesService(object):
 
     def create_invoice(self, invoice_type, inner_type):
         model = self._get_invoice_model(invoice_type)
-        return model.objects.create(type=inner_type)
+        return model.objects.create(type=inner_type, creator=self._user)
 
     def set_customer(self, invoice_type, invoice_id, customer):
 
@@ -577,7 +598,7 @@ class InvoicesService(object):
         rates = ExchangeRates.objects.get()
 
         for invoice in invoices:
-            for item in invoice.items.all():
+            for item in invoice.items.all().select_related('invoice'):
                 item.set_rates(rates)
                 result.append(item)
 
@@ -593,6 +614,50 @@ class InvoicesService(object):
             'profit_total': sum([i.profit_subtotal_uah for i in items])
         }
 
+    def handle_product_change(self, product):
+        self._handle_product_stock_change(product)
+
+    def _handle_product_stock_change(self, product):
+
+        difference = product.stock - product.initial_stock
+
+        if not difference:
+            return
+
+        if difference > 0:
+            self._create_arrival_for_product(product, difference)
+        else:
+            self._create_write_off_for_product(product, abs(difference))
+
+    def _create_arrival_for_product(self, product, qty):
+
+        arrival = self._get_todays_invoice(Arrival, Arrival.TYPE_INCOME)
+
+        arrival.add_item(product, qty=qty, update_product=False)
+
+    def _create_write_off_for_product(self, product, qty):
+
+        sale = self._get_todays_invoice(Sale, Sale.TYPE_WRITE_OFF)
+
+        sale.add_item(product, qty=qty, update_product=False)
+
+    def _get_todays_invoice(self, model, invoice_type):
+
+        today = timezone.now().date()
+        today_min = timezone.make_aware(datetime.combine(today, time.min))
+        today_max = timezone.make_aware(datetime.combine(today, time.max))
+
+        params = {
+            'creator': self._user,
+            'type': invoice_type
+        }
+
+        try:
+            return model.objects.filter(
+                created__range=(today_min, today_max), **params)[0]
+        except IndexError:
+            return model.objects.create(**params)
+
     def _get_invoice_model(self, invoice_type):
 
         models = {
