@@ -1,20 +1,21 @@
 
 CategoryTree = function (params) {
 
-    var $container = params.$container,
-        $filter = params.$filter,
-        $input = params.$input,
-        data = params.data;
+    var data = params.data,
+        $container = params.$container,
+        $list = $container.find('[data-role=categories]'),
+        $filter = $container.find('[data-role=category-filter]'),
+        $input = $container.find('[data-role=category-input]');
 
     function handleFilterKeyup() {
         var val = $(this).val();
 
         if (!val) {
-            $container.treeview('clearSearch');
+            $list.treeview('clearSearch');
             return;
         }
 
-        $container.treeview('search', [val, {
+        $list.treeview('search', [val, {
             ignoreCase: true,
             exactMatch: false,
             revealResults: true
@@ -22,17 +23,16 @@ CategoryTree = function (params) {
     }
 
     function handleSearchComplete() {
-        var $c = $container,
-            tree = $c.treeview(true);
+        var tree = $list.treeview(true);
 
         $.each(tree.getEnabled(), function () {
-            var $li = $c.find('[data-nodeid=' + this.nodeId + ']');
+            var $li = $list.find('[data-nodeid=' + this.nodeId + ']');
             $li[this.searchResult ? 'show' : 'hide']();
         })
     }
 
     function handleSearchCleared() {
-        $container.find('li').show();
+        $list.find('li').show();
     }
 
     function handleNodeSelected(event, node) {
@@ -43,7 +43,7 @@ CategoryTree = function (params) {
         $input.val('').trigger('change');
     }
 
-    $container.treeview({
+    $list.treeview({
         data: data,
         nodeIcon: 'fa fa-folder',
         highlightSearchResults: false,
