@@ -48,6 +48,10 @@ urlpatterns = [
          views.remove_item,
          name='remove-item'),
 
+    path('add-product/',
+         views.add_product,
+         name='add-product'),
+
     path('products/', views.get_products, name='products')
 
 ]
